<?php

namespace EventPortal\Domain\Event\Models;

use Carbon\Carbon;
use EventPortal\Concerns\AddToCasts;
use EventPortal\Concerns\AddToFillable;
use EventPortal\Concerns\HasStatus;
use EventPortal\Concerns\HasTeams;
use EventPortal\Concerns\Sluggable;
use EventPortal\Contracts\Event as EventContract;
use EventPortal\Contracts\Statusable;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Email\Models\EventEmail;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\ViewableStatus;
use EventPortal\Domain\Model;
use EventPortal\Domain\SignIn\Concerns\CanSignInto;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Domain\Table\Concerns\HasTable;
use EventPortal\Domain\Util\Notification\Models\EventNotification;
use EventPortal\Domain\Venue\Models\Venue;
use Iben\Statable\Statable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Notifications\Notifiable;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;
use Spatie\Sluggable\HasSlug;
use Spatie\Sluggable\SlugOptions;

class Event extends Model implements EventContract, AuditableInterface, Statusable
{
    use HasTable,
        AddToFillable,
        AddToCasts,
        Auditable,
        Statable,
        HasStatus,
        HasTeams,
        CanSignInto,
        Notifiable,
        HasSlug,
        Sluggable;

    public const KEY = 'event';

    /** @var array<string> */
    protected $fillable = [
        'name',
        'slug',
        'status',
        'email',
        'snippet',
        'description',
        'is_virtual',
        'broadcast_URL',
    ];

    /** @var string<string> */
    protected $casts = [
        'is_virtual' => 'array',
    ];

    public function getSlugOptions(): SlugOptions
    {
        return SlugOptions::create()
            ->generateSlugsFrom('name')
            ->saveSlugsTo('slug');
    }

    protected function getGraph(): string
    {
        return 'event';
    }

    public function agendas(): HasMany
    {
        return $this->hasMany(Agenda::class);
    }

    public function delegates(): BelongsToMany
    {
        return $this
            ->belongsToMany(Delegate::class)
            ->using(EventDelegate::class)
            ->withPivot(['id', 'status'])
            ->as('delegates')
            ->withTimestamps();
    }

    public function speakers(): BelongsToMany
    {
        return $this
            ->belongsToMany(Speaker::class)
            ->using(EventSpeaker::class)
            ->withPivot(['id'])
            ->as('speakers')
            ->withTimestamps();
    }

    public function venues(): BelongsToMany
    {
        return $this
            ->belongsToMany(Venue::class)
            ->using(EventVenue::class)
            ->withPivot(['id'])
            ->as('venues')
            ->withTimestamps();
    }

    public function emails(): HasMany
    {
        return $this->hasMany(EventEmail::class);
    }

    public function notifications(): HasMany
    {
        return $this->hasMany(EventNotification::class);
    }

    public function forms(): HasMany
    {
        return $this->hasMany(Form::class);
    }

    public function pathways(): HasMany
    {
        return $this->hasMany(Pathway::class);
    }

    public function transitionRouteName(): string
    {
        return 'admin.event.transition';
    }

    public function transitionRouteParameters(): array
    {
        return [
            'event' => $this->id,
        ];
    }

    public function getIsVirtualAttribute(string $json): bool
    {
        return (bool) json_decode($json)->value;
    }

    public function getEndDateAttribute(): ?Carbon
    {
        if ($this->agendas->count() <= 1) {
            return null;
        }

        return optional($this->agendas()->orderBy('date')->get()->last())->date;
    }

    public function getStartDateAttribute(): ?Carbon
    {
        return optional($this->agendas()->orderBy('date')->first())->date;
    }

    public function getReadableStartDateAttribute(): ?string
    {
        return optional($this->startDate)->format('dS F Y');
    }

    public function getReadableDatesAttribute(): string
    {
        if ($this->agendas->isEmpty()) {
            return '';
        }

        if ($this->agendas->count() <= 1) {
            return $this->startDate->format('jS F y');
        }

        return "{$this->startDate->format('jS F y')} - {$this->endDate->format('jS F y')}";
    }

    public function scopeHasDateAfterOrEqualTo(Builder $query, string $value): Builder
    {
        return $query->whereIn('id', function ($query) use ($value) {
            return $query
                ->select('event_id')
                ->from('agendas')
                ->whereDate('date', '>=', Carbon::createFromFormat('d/m/Y', $value));
        });
    }

    public function scopeHasDateBeforeOrEqualTo(Builder $query, string $value): Builder
    {
        return $query->whereIn('id', function ($query) use ($value) {
            return $query
                ->select('event_id')
                ->from('agendas')
                ->whereDate('date', '<=', Carbon::createFromFormat('d/m/Y', $value));
        });
    }

    public function scopeViewable(Builder $query): Builder
    {
        return $query->whereIn('status', ViewableStatus::get());
    }

    public function scopeForDelegate(Builder $query, Delegate $delegate): Builder
    {
        return $query->whereHas('delegates', static function (Builder $query) use ($delegate) {
            return $query
                ->where('delegate_id', $delegate->id)
                ->whereIn('status', \EventPortal\Domain\EventDelegate\ViewableStatus::get());
        });
    }

    public function scopeUpcoming(Builder $query): Builder
    {
        return $query->whereHas('agendas', static function ($query) {
            return $query->whereDate('date', '>', now());
        });
    }

    public function scopePrevious(Builder $query): Builder
    {
        return $query->whereHas('agendas', static function ($query) {
            return $query->whereDate('date', '<', now());
        });
    }
}
