<?php

namespace EventPortal\Tests\Feature\Admin\Team;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use EventPortal\Domain\Team\Models\Team;
use Illuminate\Foundation\Testing\RefreshDatabase;

class TeamUpdateTest extends TestCase
{
    use RefreshDatabase, SeedRolesAndPermissions;

    /** @test */
    public function a_team_can_be_updated()
    {
        $team = factory(Team::class)->create([
            'name' => 'Original Name',
            'description' => 'Original description',
        ]);

        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.team.update', $team), [
                'name' => 'New Name',
                'description' => 'New description',
            ])
            ->assertOk();

        $this->assertDatabaseHas('teams', [
            'name' => 'New Name',
            'description' => 'New description',
        ]);
    }

    /** @test **/
    public function the_team_name_is_required()
    {
        $team = factory(Team::class)->create([
            'name' => 'Original Name',
        ]);

        $this
            ->actingAsSuperAdmin()
            ->post(route('admin.team.update', $team), [
                'name' => null,
                'description' => 'No name',
            ])
            ->assertSessionHasErrors([
                'name',
            ]);

        $this->assertDatabaseHas('teams', [
            'name' => 'Original Name',
        ]);

        $this->assertDatabaseMissing('teams', [
            'description' => 'No name',
        ]);
    }

    /** @test **/
    public function members_can_be_updated()
    {
        $team = factory(Team::class)->create([
            'name' => 'Original Name',
        ]);
        [$teamMember1, $teamMember2] = factory(Admin::class, 2)->create();

        $team->admins()->sync($teamMember1->id);

        $this
            ->actingAsSuperAdmin()
            ->followingRedirects()
            ->post(route('admin.team.update', $team), [
                'name' => 'New Team',
                'members' => [$teamMember1->id]
            ]);

        $team = Team::first();
        $teamUserIds = $team->admins->pluck('id');

        $this->assertCount(1, $team->admins);
        $this->assertContains($teamMember1->id, $teamUserIds);
        $this->assertNotContains($teamMember2->id, $teamUserIds);
    }

    /** @test **/
    public function only_valid_members_can_be_added()
    {
        $this->markTestSkipped('Need to complete');

        $admin = factory(Admin::class)->create();

        $team = factory(Team::class)->create([
            'name' => 'Original Name',
        ]);
        [$teamMember1, $teamMember2] = factory(Admin::class, 2)->create();

        $team->admins()->sync([$teamMember1->id, $teamMember2->id]);

        $this
            ->actingAsSuperAdmin($admin)
            ->post(route('admin.team.store'), [
                'name' => 'New Team',
                'members' => [
                    321,
                    123,
                ],
            ])
            ->assertSessionHasErrors([
                'members',
            ]);

        $team = Team::first();
        $teamUserIds = $team->admins->pluck('id');

        $this->assertCount(2, $team->admins);
        $this->assertContains($teamMember1->id, $teamUserIds);
        $this->assertContains($teamMember2->id, $teamUserIds);
        $this->assertNotContains($admin->id, $teamUserIds);
    }
}
